/* Copyright (c) 2013-2018. The SimGrid Team. All rights reserved.          */

/* This program is free software; you can redistribute it and/or modify it
 * under the terms of the license (GNU LGPL) which comes with this package. */

#ifndef KERNEL_ROUTING_NETPOINT_HPP_
#define KERNEL_ROUTING_NETPOINT_HPP_

#include <xbt/Extendable.hpp>
#include <xbt/base.h>
#include <xbt/signal.hpp>

#include <simgrid/kernel/routing/NetZoneImpl.hpp>

namespace simgrid {
namespace kernel {
namespace routing {

/** @ingroup ROUTING_API
 *  @brief Network cards are the vertices in the graph representing the network, used to compute paths between nodes.
 *
 * @details This represents a position in the network. One can send information between two netpoints
 */
class NetPoint : public simgrid::xbt::Extendable<NetPoint> {

public:
  enum class Type { Host, Router, NetZone };

  NetPoint(std::string name, NetPoint::Type componentType, NetZoneImpl* netzone_p);
  ~NetPoint() = default;

  // Our rank in the vertices_ array of the netzone that contains us.
  unsigned int id() { return id_; }
  const std::string& getName() const { return name_; }
  const char* getCname() const { return name_.c_str(); }
  /** @brief the NetZone in which this NetPoint is included */
  NetZoneImpl* netzone() { return netzone_; }

  bool isNetZone() { return component_type_ == Type::NetZone; }
  bool isHost() { return component_type_ == Type::Host; }
  bool isRouter() { return component_type_ == Type::Router; }

  static simgrid::xbt::signal<void(NetPoint*)> onCreation;

  bool operator<(const NetPoint& rhs) const { return name_ < rhs.name_; }

private:
  unsigned int id_;
  std::string name_;
  NetPoint::Type component_type_;
  NetZoneImpl* netzone_;
};
} // namespace routing
} // namespace kernel
} // namespace simgrid

XBT_PUBLIC sg_netpoint_t sg_netpoint_by_name_or_null(const char* name);

#endif /* KERNEL_ROUTING_NETPOINT_HPP_ */
